data = {
    "report_id": "4455bf",
    "website_link": "https://www.an-old-website.com/",
    "total_links": 17,
    "internal_links": [
        ["/about", "0.34s"],
        ["/contact", "0.23s"],
        ["/blog", "0.56s"],
        ["/blog/2020/07/12/why-is-python-preferred-for-automation.html", "0.72s"],
        ["/blog/2020/07/15/is-nim-a-python-killer.html", "0.66s"],
        ["/blog/2020/07/16/python-and-javascript-best-of-both-worlds.html", "0.89s"],
        ["/favicon.ico", "0.22s"],
        ["/assets/images/logo.jpg", "0.34s"],
        ["/assets/images/banner.jpg", "0.56s"],
        ["/assets/css/main.css", "0.33s"],
    ],
    "external_links": [
        ["https://ajax.googleapis.com/ajax/libs/jquery/3.5.1/jquery.min.js", "1.23s"],
        [
            "https://maxcdn.bootstrapcdn.com/bootstrap/3.4.1/js/bootstrap.min.js",
            "0.97s",
        ],
        [
            "https://maxcdn.bootstrapcdn.com/bootstrap/3.4.1/css/bootstrap.min.css",
            "0.92s",
        ],
        ["https://search.google.com", "1.07s"],
    ],
    "broken_links": [
        [
            "/blog/2020/07/10/test-automation-techniques-using-python.html",
            "404_NOT_FOUND",
        ],
        ["https://www.faecbook.com/user/ElectricBoa", "404_NOT_FOUND"],
        ["../pelican-installation-procedure.html", "404_NOT_FOUND"],
    ],
    "data_time": "2019/12/22 11:30:00 IST",
}
